/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

using Gtk;

namespace Yggtk {

    public class Application : Gtk.Application {

        public Application () {
            Object (
                application_id: "org.yggtk.yggtk",
                flags: ApplicationFlags.FLAGS_NONE
            );
            add_actions ();
        }

        private void add_actions () {
            SimpleAction update_yggtk_action = new SimpleAction ("update-yggtk", null);
            update_yggtk_action.activate.connect (update_yggtk);
            add_action (update_yggtk_action);

            SimpleAction update_yggdrasil_action = new SimpleAction ("update-yggdrasil", null);
            update_yggdrasil_action.activate.connect (update_yggdrasil);
            add_action (update_yggdrasil_action);
        }

        private void update_yggtk () {
            try {
                unowned string repo_url = Environment.get_variable ("YGG_UPDATE_REPO");
                string temp_dir = Utils.mktempdir ();

                if (repo_url == null) {
                    repo_url = "https://git.macaw.me/plant_1312/Yggtk.git";
                }

                Process.spawn_command_line_sync (@"git clone $repo_url $temp_dir");
                Process.spawn_command_line_sync (@"meson $temp_dir/build $temp_dir");
                Process.spawn_command_line_sync (@"ninja -C $temp_dir/build");
                Process.spawn_command_line_sync (@"cp $temp_dir/build/src/yggtk .");
                Process.spawn_command_line_sync (@"rm -rf $temp_dir");
                Process.spawn_command_line_sync ("./yggtk");

                Gtk.main_quit ();
            } catch (Error e) {
                print ("Error update: %s\n", e.message);
            }
        }

        private void update_yggdrasil () {
            try {
                string pm;
                Process.spawn_command_line_sync ("pkexec pacman -S yggdrasil --noconfirm", out pm);

                if (pm == "Cannot run program pacman: No such file or directory\n") {
                    Process.spawn_command_line_sync (
                        "wget https://2375-115685026-gh.circle-artifacts.com/0/yggdrasil-0.3.12-amd64.deb"
                    );
                    Process.spawn_command_line_sync ("pkexec dpkg -i yggdrasil-0.3.12-amd64.deb");
                }
            } catch (Error e) {
                print ("Error download: %s\n", e.message);
            }
        }

        protected override void activate () {
            AppWindow window = new AppWindow (this);
            window.show ();
        }

        public static int main (string[] args) {
            Application app = new Application ();
            return app.run (args);
        }

    }

}